@@ -4,6 +4,8 @@ from flask import request
 
 from sqlalchemy_blender import QueryBuffer
 from sqlalchemy_blender.helpers import iserialize
+from sqlalchemy_blender.helpers import tojson
+from sqlalchemy_blender import helpers
 
 from handyhttp.responses import HTTPSuccess
 from handyhttp.responses import HTTPCreated
@@ -30,20 +32,35 @@ class Routes:
         self.key = key
         self.throw = throw
         self.response = response
+        self.errors = list()
+
+    def refresh(self):
+        self.errors.clear()
 
     def payload(self):
         return json.loads(request.data)
 
-    def exception(self, exception):
-        raise exception
+    def bake(self, http, data=None, **kwargs):
+        return http(data, **self.metadata(**kwargs, **dict(metadata=dict(count=len(data or [])))), **kwargs)
+
+    # def exception(self, exception):
+    #     raise exception
+
+    def metadata(self, **kwargs):
+        resp = dict()
+        resp['links'] = {}
+        resp['links']['self'] = request.url
+        resp['links']['_self'] = request.base_url
+        resp['metadata'] = dict(args=dict(request.args))
+        resp['errors'] = self.errors
+
+        if kwargs:
+            resp.update(kwargs)
+        return resp
 
     def json(self, data, *args, **kwargs):
-        if isinstance(data, list) and len(data) and isinstance(data[0], tuple):
-            _json = []
-            for item in data:
-                _json.append({key: getattr(item, key, None) for key in item._fields})
-            return _json
-        return iserialize(data, **kwargs)
+        resp = tojson(data, *args, **kwargs)
+        return resp
 
     @link(url='', methods=['GET'])
     def get(self, *args, **kwargs):
@@ -62,8 +79,9 @@ class Routes:
         :rtype: HTTPSuccess
         """
 
-        query = QueryBuffer(self.model).all()
-        return self.response(HTTPSuccess(self.json(query.data, **query.queryargs.__dict__)))
+        query = QueryBuffer(self.model).apply().all()
+        self.errors.extend(query.errors)
+        return self.bake(HTTPSuccess, self.json(query.data, **query.queryargs.__dict__))
 
     @link(url='/<resource>', methods=['HEAD'])
     def head(self, resource):
@@ -71,8 +89,8 @@ class Routes:
         resp = query.one(self.key, resource)
 
         if not resp:
-            return self.exception(HTTPNotFound())
-        return HTTPSuccess()
+            raise HTTPNotFound()
+        return self.bake(HTTPSuccess)
 
     @link(url='/<resource>', methods=['GET'])
     def one(self, resource, *args, **kwargs):
@@ -100,8 +118,8 @@ class Routes:
         resp = query.one(self.key, resource)
 
         if not resp.data:
-            return self.exception(HTTPNotFound())
-        return self.response(HTTPSuccess(self.json(query.data, **query.queryargs.__dict__)))
+            raise HTTPNotFound(f'{self.model.__tablename__} not found')
+        return self.bake(HTTPSuccess, self.json(query.data, **query.queryargs.__dict__))
 
     @link(url='/<resource>/<path:field>', methods=['GET'])
     def one_child_resource(self, resource, field, *args, **kwargs):
@@ -111,7 +129,7 @@ class Routes:
         prop = None
 
         if not instance:
-            return self.exception(HTTPNotFound())
+            raise HTTPNotFound(f'{self.model.__tablename__} not found')
 
         if len(path) > 1:
             child = None
@@ -130,16 +148,17 @@ class Routes:
             prop = field
             data = getattr(instance, field)
 
-        return HTTPSuccess({prop: data}, path=field.split('/'))
+        resp = {prop: data}
+        return self.bake(HTTPSuccess, self.json(resp, **query.queryargs.__dict__))
 
     @link(url='/', methods=['POST'])
     def post(self, *args, **kwargs):
         payload = request.json
         if not payload:
-            self.exception(HTTPNotProcessable(msg='Invalid request payload.'))
+            raise HTTPNotProcessable(msg='Invalid request payload.')
         instance = self.dao.create(payload, **kwargs)
-        self.dao.save(instance)
-        return HTTPCreated(self.json(instance))
+        # self.dao.save(HTTPCreated)
+        return self.bake(HTTPSuccess, self.json(instance))
 
     @link(url='/<resource>/<field>/', methods=['POST'])
     def post_child_resource(self, resource, field, *args, **kwargs):
@@ -147,7 +166,7 @@ class Routes:
         child = getattr(self.model.__mapper__.relationships, field).entity.entity(**self.payload())
         getattr(instance, field).append(child)
         self.dao.save(instance)
-        return HTTPUpdated()
+        return self.bake(HTTPSuccess, self.json(instance))
 
     @link(url='/<resource>', methods=['DELETE'])
     def delete(self, resource, *args, **kwargs):
@@ -157,7 +176,7 @@ class Routes:
 
         if self.dao:
             self.dao.delete(instance.data)
-        return self.response(HTTPSuccess())
+        return self.bake(HTTPDeleted)
 
     @link(url='/<resource>', methods=['PUT'])
     def put(self, resource, *args, **kwargs):
@@ -165,4 +184,4 @@ class Routes:
         instance = query.data
         payload = self.payload()
         self.dao.update(instance, payload)
-        return self.response(HTTPUpdated(iserialize(instance)))
+        return self.bake(HTTPUpdated, self.json(instance))
